@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -23,6 +24,7 @@
 
 #include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 #include "DicomWebServers.h"
+#include <DicomFormat/DicomMap.h>
 
 #include <Compatibility.h>
 #include <Logging.h>
@@ -36,7 +38,8 @@
 
 // Assume Latin-1 encoding by default (as in the Orthanc core)
 static Orthanc::Encoding defaultEncoding_ = Orthanc::Encoding_Latin1;
-static std::unique_ptr<OrthancPlugins::OrthancConfiguration> configuration_;
+static std::unique_ptr<OrthancPlugins::OrthancConfiguration> dicomWebConfiguration_;
+static std::unique_ptr<OrthancPlugins::OrthancConfiguration> globalConfiguration_;
 static bool serversInDatabase_ = false;
 static const int32_t GLOBAL_PROPERTY_SERVERS = 5468;
 
@@ -64,37 +67,6 @@ namespace OrthancPlugins
   }
 
 
-  void ParseContentType(std::string& application,
-                        std::map<std::string, std::string>& attributes,
-                        const std::string& header)
-  {
-    application.clear();
-    attributes.clear();
-
-    std::vector<std::string> tokens;
-    Orthanc::Toolbox::TokenizeString(tokens, header, ';');
-
-    assert(tokens.size() > 0);
-    application = tokens[0];
-    Orthanc::Toolbox::StripSpaces(application);
-    Orthanc::Toolbox::ToLowerCase(application);
-
-    boost::regex pattern("\\s*([^=]+)\\s*=\\s*(([^=\"]+)|\"([^=\"]+)\")\\s*");
-    
-    for (size_t i = 1; i < tokens.size(); i++)
-    {
-      boost::cmatch what;
-      if (boost::regex_match(tokens[i].c_str(), what, pattern))
-      {
-        std::string key(what[1]);
-        std::string value(what.length(3) != 0 ? what[3] : what[4]);
-        Orthanc::Toolbox::ToLowerCase(key);
-        attributes[key] = value;
-      }
-    }
-  }
-
-
   void ParseAssociativeArray(std::map<std::string, std::string>& target,
                              const Json::Value& value)
   {
@@ -292,20 +264,47 @@ namespace OrthancPlugins
 
   namespace Configuration
   {
+    // TODO: find a way to reuse/share the method from/with the Orthanc Core
+    void LoadMainDicomTags(const Json::Value& configuration)
+    {
+      static const char* const EXTRA_MAIN_DICOM_TAGS = "ExtraMainDicomTags";
+
+      // note: the configuration is assumed to be valid since it has already been parsed by the Orthanc Core
+      Json::Value::Members levels(configuration[EXTRA_MAIN_DICOM_TAGS].getMemberNames());
+
+      for (Json::Value::ArrayIndex i = 0; i < levels.size(); i++)
+      {
+        Orthanc::ResourceType level = Orthanc::StringToResourceType(levels[i].c_str());
+
+        const Json::Value& content = configuration[EXTRA_MAIN_DICOM_TAGS][levels[i]];
+
+        if (content.size() > 0)
+        {
+          for (Json::Value::ArrayIndex t = 0; t < content.size(); t++)
+          {
+            const std::string& tagName = content[t].asString();
+            Orthanc::DicomTag tag(0, 0);
+            OrthancPlugins::ParseTag(tag, tagName);
+            Orthanc::DicomMap::AddMainDicomTag(tag, level);
+          }
+        }
+      }
+    }
+
     void Initialize()
     {
-      configuration_.reset(new OrthancConfiguration);
-      
-      OrthancConfiguration global;
-      global.GetSection(*configuration_, "DicomWeb");
+      dicomWebConfiguration_.reset(new OrthancConfiguration);
+      globalConfiguration_.reset(new OrthancConfiguration);
+
+      globalConfiguration_->GetSection(*dicomWebConfiguration_, "DicomWeb");
 
       std::string s;
-      if (global.LookupStringValue(s, "DefaultEncoding"))
+      if (globalConfiguration_->LookupStringValue(s, "DefaultEncoding"))
       {
         defaultEncoding_ = Orthanc::StringToEncoding(s.c_str());
       }
 
-      if (!configuration_->LookupBooleanValue(serversInDatabase_, "ServersInDatabase"))
+      if (!dicomWebConfiguration_->LookupBooleanValue(serversInDatabase_, "ServersInDatabase"))
       {
         serversInDatabase_ = false;
       }
@@ -328,45 +327,46 @@ namespace OrthancPlugins
       std::set<Orthanc::DicomTag> tags;
       GetExtrapolatedMetadataTags(tags, Orthanc::ResourceType_Study);
       GetExtrapolatedMetadataTags(tags, Orthanc::ResourceType_Series);
+
+      LoadMainDicomTags(globalConfiguration_->GetJson());
     }
 
 
     std::string GetStringValue(const std::string& key,
                                const std::string& defaultValue)
     {
-      assert(configuration_.get() != NULL);
-      return configuration_->GetStringValue(key, defaultValue);
+      assert(dicomWebConfiguration_.get() != NULL);
+      return dicomWebConfiguration_->GetStringValue(key, defaultValue);
     }
 
 
     bool GetBooleanValue(const std::string& key,
                          bool defaultValue)
     {
-      assert(configuration_.get() != NULL);
-      return configuration_->GetBooleanValue(key, defaultValue);
+      assert(dicomWebConfiguration_.get() != NULL);
+      return dicomWebConfiguration_->GetBooleanValue(key, defaultValue);
     }
 
 
     bool LookupBooleanValue(bool& target,
                             const std::string& key)
     {
-      assert(configuration_.get() != NULL);
-      return configuration_->LookupBooleanValue(target, key);
+      assert(dicomWebConfiguration_.get() != NULL);
+      return dicomWebConfiguration_->LookupBooleanValue(target, key);
     }
 
 
     unsigned int GetUnsignedIntegerValue(const std::string& key,
                                          unsigned int defaultValue)
     {
-      assert(configuration_.get() != NULL);
-      return configuration_->GetUnsignedIntegerValue(key, defaultValue);
+      assert(dicomWebConfiguration_.get() != NULL);
+      return dicomWebConfiguration_->GetUnsignedIntegerValue(key, defaultValue);
     }
 
-
-    std::string GetDicomWebRoot()
+    std::string GetRootPath(const char* configName, const char* defaultValue)
     {
-      assert(configuration_.get() != NULL);
-      std::string root = configuration_->GetStringValue("Root", "/dicom-web/");
+      assert(dicomWebConfiguration_.get() != NULL);
+      std::string root = dicomWebConfiguration_->GetStringValue(configName, defaultValue);
 
       // Make sure the root URI starts and ends with a slash
       if (root.size() == 0 ||
@@ -383,7 +383,17 @@ namespace OrthancPlugins
       return root;
     }
 
-    
+    std::string GetDicomWebRoot()
+    {
+      return GetRootPath("Root", "/dicom-web/");
+    }
+
+    std::string GetPublicRoot()
+    {
+      std::string root = GetDicomWebRoot();
+      return GetRootPath("PublicRoot", root.c_str());
+    }
+
     std::string GetOrthancApiRoot()
     {
       std::string root = Configuration::GetDicomWebRoot();
@@ -420,8 +430,8 @@ namespace OrthancPlugins
 
     std::string GetWadoRoot()
     {
-      assert(configuration_.get() != NULL);
-      std::string root = configuration_->GetStringValue("WadoRoot", "/wado/");
+      assert(dicomWebConfiguration_.get() != NULL);
+      std::string root = dicomWebConfiguration_->GetStringValue("WadoRoot", "/wado/");
 
       // Make sure the root URI starts with a slash
       if (root.size() == 0 ||
@@ -476,11 +486,23 @@ namespace OrthancPlugins
     }
 
 
-    std::string GetBaseUrl(const HttpClient::HttpHeaders& headers)
+    std::string GetBasePublicUrl(const HttpClient::HttpHeaders& headers)
     {
-      assert(configuration_.get() != NULL);
-      std::string host = configuration_->GetStringValue("Host", "");
-      bool https = configuration_->GetBooleanValue("Ssl", false);
+      assert(dicomWebConfiguration_.get() != NULL);
+      std::string host = dicomWebConfiguration_->GetStringValue("Host", "");
+      bool https = dicomWebConfiguration_->GetBooleanValue("Ssl", false);
+
+      std::string forwardedHost, forwardedProto;
+      if (host.empty() &&
+          LookupHttpHeader2(forwardedHost, headers, "x-forwarded-host") &&
+          LookupHttpHeader2(forwardedProto, headers, "x-forwarded-proto"))
+      {
+        // There is a "X-Forwarded-Proto" and a "X-Forwarded-Host" HTTP header in the query
+        // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Forwarded-Proto
+        
+        host = Orthanc::Toolbox::StripSpaces(forwardedHost);
+        https = IsHttpsProto(Orthanc::Toolbox::StripSpaces(forwardedProto), https);
+      }
 
       std::string forwarded;
       if (host.empty() &&
@@ -530,11 +552,10 @@ namespace OrthancPlugins
         host = "localhost:8042";
       }
 
-      return (https ? "https://" : "http://") + host + GetDicomWebRoot();
+      return (https ? "https://" : "http://") + host + GetPublicRoot();
     }
 
-
-    std::string GetBaseUrl(const OrthancPluginHttpRequest* request)
+    std::string GetBasePublicUrl(const OrthancPluginHttpRequest* request)
     {
       HttpClient::HttpHeaders headers;
 
@@ -549,7 +570,22 @@ namespace OrthancPlugins
         headers["Host"] = value;
       }
 
-      return GetBaseUrl(headers);
+      if (LookupHttpHeader(value, request, "x-forwarded-host"))
+      {
+        headers["X-Forwarded-Host"] = value;
+      }
+
+      if (LookupHttpHeader(value, request, "x-forwarded-proto"))
+      {
+        headers["X-Forwarded-Proto"] = value;
+      }
+
+      if (LookupHttpHeader(value, request, "host"))
+      {
+        headers["Host"] = value;
+      }
+
+      return GetBasePublicUrl(headers);
     }
 
 
@@ -635,6 +671,11 @@ namespace OrthancPlugins
       }
     }
 
+    unsigned int GetMetadataWorkerThreadsCount()
+    {
+      return GetUnsignedIntegerValue("MetadataWorkerThreadsCount", 4);
+    }
+
     
     MetadataMode GetMetadataMode(Orthanc::ResourceType level)
     {
@@ -688,7 +729,7 @@ namespace OrthancPlugins
 
       std::list<std::string> s;
       
-      if (configuration_->LookupListOfStrings(s, key, false))
+      if (dicomWebConfiguration_->LookupListOfStrings(s, key, false))
       {
         for (std::list<std::string>::const_iterator it = s.begin(); it != s.end(); ++it)
         {
@@ -755,7 +796,7 @@ namespace OrthancPlugins
       else
       {
         OrthancConfiguration servers;
-        configuration_->GetSection(servers, "Servers");
+        dicomWebConfiguration_->GetSection(servers, "Servers");
         DicomWebServers::GetInstance().LoadGlobalConfiguration(servers.GetJson());
       }
     }
