@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,18 +22,19 @@
 
 #include "DicomWebClient.h"
 
+#include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 #include "DicomWebServers.h"
 
-#include <json/reader.h>
+#include <ChunkedBuffer.h>
+#include <Compatibility.h>
+#include <HttpServer/MultipartStreamReader.h>
+#include <Logging.h>
+#include <Toolbox.h>
+
 #include <list>
 #include <set>
 #include <boost/lexical_cast.hpp>
 
-#include <Core/HttpServer/MultipartStreamReader.h>
-#include <Core/ChunkedBuffer.h>
-#include <Core/Toolbox.h>
-#include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
-
 
 #include <boost/thread.hpp>
 #include <boost/algorithm/string/predicate.hpp>
@@ -56,7 +58,7 @@ public:
     }
 
     void SetContent(const std::string& key,
-                    const std::string& value)
+                    const Json::Value& value)
     {
       that_.SetContent(key, value);
     }
@@ -134,7 +136,7 @@ private:
 
   boost::mutex                  mutex_;
   FunctionResult                functionResult_;  // Can only be modified by the "Worker()" function
-  std::auto_ptr<boost::thread>  worker_;
+  std::unique_ptr<boost::thread>  worker_;
   Json::Value                   content_;
   IFunctionFactory*             factory_;
   bool                          stopping_;
@@ -165,8 +167,16 @@ private:
     }
   }
 
+  // void SetContent(const std::string& key,
+  //                 const std::string& value)
+  // {
+  //   boost::mutex::scoped_lock lock(mutex_);
+  //   content_[key] = value;
+  //   UpdateContent(content_);
+  // }
+
   void SetContent(const std::string& key,
-                  const std::string& value)
+                  const Json::Value& value)
   {
     boost::mutex::scoped_lock lock(mutex_);
     content_[key] = value;
@@ -178,11 +188,11 @@ private:
   {
     assert(job != NULL && factory != NULL);
 
-    JobContext context(*job);
-
     try
     {
-      std::auto_ptr<IFunction> function(factory->CreateFunction());
+      JobContext context(*job);
+
+      std::unique_ptr<IFunction> function(factory->CreateFunction());
       function->Execute(context);
 
       {
@@ -245,7 +255,7 @@ public:
     }
   }
 
-  virtual OrthancPluginJobStepStatus Step()
+  virtual OrthancPluginJobStepStatus Step() ORTHANC_OVERRIDE
   {
     if (factory_ == NULL)
     {
@@ -279,7 +289,7 @@ public:
     }
   }
 
-  virtual void Stop(OrthancPluginJobStopReason reason)
+  virtual void Stop(OrthancPluginJobStopReason reason) ORTHANC_OVERRIDE
   {
     if (factory_ == NULL)
     {
@@ -306,7 +316,7 @@ public:
     }
   }
 
-  virtual void Reset()
+  virtual void Reset() ORTHANC_OVERRIDE
   {
     boost::mutex::scoped_lock lock(mutex_);
 
@@ -329,7 +339,7 @@ static void SubmitJob(OrthancPluginRestOutput* output,
                       const Json::Value& body,
                       bool defaultSynchronous)
 {
-  std::auto_ptr<OrthancPlugins::OrthancJob> protection(job);
+  std::unique_ptr<OrthancPlugins::OrthancJob> protection(job);
 
   bool synchronous;
 
@@ -490,8 +500,40 @@ static void CheckStowAnswer(const Json::Value& response,
 }
 
 
+// static void AddResourceForJobContent(Json::Value resourcesForJobContent /* out */, const char* resourceType, const std::string& resourceId)
+static void AddResourceForJobContent(Json::Value& resourcesForJobContent /* out */, Orthanc::ResourceType resourceType, const std::string& resourceId)
+{
+  // const char* resourceGroup = "Instances";
+  // if (resourceType == "Study")
+  // {
+  //   resourceGroup = "Studies";
+  // }
+  // else if (resourceType == "Series")
+  // {
+  //   resourceGroup = "Series";
+  // }
+  // else if (resourceType == "Patient")
+  // {
+  //   resourceGroup = "Patients";
+  // }
+  // else if (resourceType == "Instance")
+  // {
+  //   resourceGroup = "Instances";
+  // }
+  const char* resourceGroup = Orthanc::GetResourceTypeText(resourceType, true, true);
+
+  if (!resourcesForJobContent.isMember(resourceGroup))
+  {
+    resourcesForJobContent[resourceGroup] = Json::arrayValue;
+  }
+  
+  resourcesForJobContent[resourceGroup].append(resourceId);
+}
+
+
 static void ParseStowRequest(std::list<std::string>& instances /* out */,
                              std::map<std::string, std::string>& httpHeaders /* out */,
+                             Json::Value& resourcesForJobContent /* out */,
                              const Json::Value& body /* in */)
 {
   static const char* RESOURCES = "Resources";
@@ -527,27 +569,30 @@ static void ParseStowRequest(std::list<std::string>& instances /* out */,
     }
 
     // Test whether this resource is an instance
-    Json::Value tmp;
-    if (OrthancPlugins::RestApiGet(tmp, "/instances/" + resource, false))
+    Json::Value tmpResource;
+    Json::Value tmpInstances;
+    if (OrthancPlugins::RestApiGet(tmpResource, "/instances/" + resource, false))
     {
-      AddInstance(instances, tmp);
+      AddInstance(instances, tmpResource);
+      AddResourceForJobContent(resourcesForJobContent, Orthanc::ResourceType_Instance, resource);
     }
     // This was not an instance, successively try with series/studies/patients
-    else if ((OrthancPlugins::RestApiGet(tmp, "/series/" + resource, false) &&
-              OrthancPlugins::RestApiGet(tmp, "/series/" + resource + "/instances", false)) ||
-             (OrthancPlugins::RestApiGet(tmp, "/studies/" + resource, false) &&
-              OrthancPlugins::RestApiGet(tmp, "/studies/" + resource + "/instances", false)) ||
-             (OrthancPlugins::RestApiGet(tmp, "/patients/" + resource, false) &&
-              OrthancPlugins::RestApiGet(tmp, "/patients/" + resource + "/instances", false)))
+    else if ((OrthancPlugins::RestApiGet(tmpResource, "/series/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmpInstances, "/series/" + resource + "/instances", false)) ||
+             (OrthancPlugins::RestApiGet(tmpResource, "/studies/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmpInstances, "/studies/" + resource + "/instances", false)) ||
+             (OrthancPlugins::RestApiGet(tmpResource, "/patients/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmpInstances, "/patients/" + resource + "/instances", false)))
     {
-      if (tmp.type() != Json::arrayValue)
+      if (tmpInstances.type() != Json::arrayValue)
       {
         throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
       }
 
-      for (Json::Value::ArrayIndex j = 0; j < tmp.size(); j++)
+      for (Json::Value::ArrayIndex j = 0; j < tmpInstances.size(); j++)
       {
-        AddInstance(instances, tmp[j]);
+        AddInstance(instances, tmpInstances[j]);
+        AddResourceForJobContent(resourcesForJobContent, Orthanc::StringToResourceType(tmpResource["Type"].asString().c_str()), resource);
       }
     }
     else
@@ -579,6 +624,7 @@ private:
   Action                                   action_;
   size_t                                   networkSize_;
   bool                                     debug_;
+  Json::Value                              resourcesForJobContent_;
 
   bool ReadNextInstance(std::string& dicom,
                         JobContext& context)
@@ -622,6 +668,7 @@ private:
     JobContext&     context_;
     std::string     boundary_;
     bool            done_;
+    size_t          processedSize_;
 
   public:
     RequestBody(StowClientJob& that,
@@ -629,11 +676,12 @@ private:
       that_(that),
       context_(context),
       boundary_(that.boundary_),
-      done_(false)
+      done_(false),
+      processedSize_(0)
     {
     }
 
-    virtual bool ReadNextChunk(std::string& chunk)
+    virtual bool ReadNextChunk(std::string& chunk) ORTHANC_OVERRIDE
     {
       if (done_)
       {
@@ -659,9 +707,16 @@ private:
 
         //boost::this_thread::sleep(boost::posix_time::seconds(1));
 
+        processedSize_ += chunk.size();
+        
         return true;
       }
     }
+
+    size_t GetProcessedSize() const
+    {
+      return processedSize_;
+    }
   };
 
 
@@ -682,12 +737,13 @@ private:
       size_t startPosition;
 
       // The lifetime of "body" should be larger than "client"
-      std::auto_ptr<RequestBody> body;
-      std::auto_ptr<OrthancPlugins::HttpClient> client;
+      std::unique_ptr<RequestBody> body;
+      std::unique_ptr<OrthancPlugins::HttpClient> client;
 
       {
         boost::mutex::scoped_lock lock(that_.mutex_);
         context.SetContent("InstancesCount", boost::lexical_cast<std::string>(that_.instances_.size()));
+        context.SetContent("Resources", that_.GetResourcesForJobContent());
         serverName = that_.serverName_;
         
         startPosition = that_.position_;        
@@ -703,6 +759,7 @@ private:
       OrthancPlugins::HttpClient::HttpHeaders answerHeaders;
       Json::Value answerBody;
 
+      assert(client.get() != NULL);
       client->SetBody(*body);
 
       try
@@ -716,10 +773,28 @@ private:
           /**
            * "Length required" error. This might indicate an older
            * version of Orthanc (<= 1.5.6) that does not support
-           * chunked transfers.
+           * chunked transfers, or a version of Orthanc <= 1.7.2 that
+           * supports chunk transfers, but cannot receive multipart
+           * messages larger than 2GB. The latter problem is fixed by:
+           * https://hg.orthanc-server.com/orthanc/rev/36257d6f348f
            **/
-          LOG(ERROR) << "The remote DICOMweb server \"" << serverName << "\" does not support chunked transfers, "
-                     << "set configuration option \"ChunkedTransfers\" to \"0\" in the configuration";
+          if (client->IsChunkedTransfersAllowed())
+          {
+            LOG(ERROR) << "The remote DICOMweb server \"" << serverName << "\" does not support chunked transfers "
+                       << "(this might indicate Orthanc <= 1.5.6), set configuration option \"ChunkedTransfers\" "
+                       << "to \"false\" in the configuration (or upgrade remote Orthanc if applicable)";
+          }
+          else if (body->GetProcessedSize() >= 2 * static_cast<uint64_t>(1024 * 1024 * 1024))
+          {
+            LOG(ERROR) << "Cannot send a study larger than 2GB (chunked transfer is disabled) using STOW-RS, "
+                       << "this might indicate that the remote DICOMweb server is Orthanc <= 1.7.2 "
+                       << "(if so, please upgrade the remote Orthanc)";
+          }
+          else
+          {
+            LOG(ERROR) << "Cannot send a study of " << (body->GetProcessedSize() / (1024 * 1024))
+                       << "MB with STOW-RS (chunked transfer is disabled), check out the logs of the remote modality";
+          }
         }
 
         throw;
@@ -739,21 +814,21 @@ private:
   };
   
 
-  virtual void CancelFunction()
+  virtual void CancelFunction() ORTHANC_OVERRIDE
   {
     boost::mutex::scoped_lock lock(mutex_);
     action_ = Action_Cancel;
   }
   
 
-  virtual void PauseFunction()
+  virtual void PauseFunction() ORTHANC_OVERRIDE
   {
     boost::mutex::scoped_lock lock(mutex_);
     action_ = Action_Pause;
   }
 
   
-  virtual IFunction* CreateFunction()
+  virtual IFunction* CreateFunction() ORTHANC_OVERRIDE
   {
     action_ = Action_None;
     return new F(*this);
@@ -763,14 +838,16 @@ private:
 public:
   StowClientJob(const std::string& serverName,
                 const std::list<std::string>& instances,
-                const OrthancPlugins::HttpClient::HttpHeaders& headers) :
+                const OrthancPlugins::HttpClient::HttpHeaders& headers,
+                const Json::Value& resourcesForJobContent) :
     SingleFunctionJob("DicomWebStowClient"),
     serverName_(serverName),
     headers_(headers),
     position_(0),
     action_(Action_None),
     networkSize_(0),
-    debug_(false)
+    debug_(false),
+    resourcesForJobContent_(resourcesForJobContent)
   {
     SetFactory(*this);
 
@@ -806,6 +883,11 @@ public:
   {
     debug_ = debug;
   }
+
+  const Json::Value& GetResourcesForJobContent()
+  {
+    return resourcesForJobContent_;
+  }
 };
 
 
@@ -834,12 +916,13 @@ void StowClient(OrthancPluginRestOutput* output,
 
   std::list<std::string> instances;
   std::map<std::string, std::string> httpHeaders;
-  ParseStowRequest(instances, httpHeaders, body);
+  Json::Value resourcesForJobContent;
+  ParseStowRequest(instances, httpHeaders, resourcesForJobContent, body);
 
   OrthancPlugins::LogInfo("Sending " + boost::lexical_cast<std::string>(instances.size()) +
                           " instances using STOW-RS to DICOMweb server: " + serverName);
 
-  std::auto_ptr<StowClientJob> job(new StowClientJob(serverName, instances, httpHeaders));
+  std::unique_ptr<StowClientJob> job(new StowClientJob(serverName, instances, httpHeaders, resourcesForJobContent));
 
   bool debug;
   if (OrthancPlugins::LookupBooleanValue(debug, body, "Debug"))
@@ -980,12 +1063,12 @@ private:
   boost::mutex                                   mutex_;
   State                                          state_;
   std::list<std::string>                         instances_;
-  std::auto_ptr<Orthanc::MultipartStreamReader>  reader_;
+  std::unique_ptr<Orthanc::MultipartStreamReader>  reader_;
   uint64_t                                       networkSize_;
 
   virtual void HandlePart(const Orthanc::MultipartStreamReader::HttpHeaders& headers,
                           const void* part,
-                          size_t size)
+                          size_t size) ORTHANC_OVERRIDE
   {
     std::string contentType;
     if (!Orthanc::MultipartStreamReader::GetMainContentType(contentType, headers))
@@ -1059,7 +1142,7 @@ public:
   }
 
   virtual void AddHeader(const std::string& key,
-                         const std::string& value)
+                         const std::string& value) ORTHANC_OVERRIDE
   {
     boost::mutex::scoped_lock lock(mutex_);
 
@@ -1108,7 +1191,7 @@ public:
   }
 
   virtual void AddChunk(const void* data,
-                        size_t size)
+                        size_t size) ORTHANC_OVERRIDE
   {
     boost::mutex::scoped_lock lock(mutex_);
 
@@ -1228,7 +1311,7 @@ private:
   std::vector<Resource*>  resources_;
   bool                    stopped_;
   std::list<std::string>  retrievedInstances_;
-  std::auto_ptr<WadoRetrieveAnswer>  answer_;
+  std::unique_ptr<WadoRetrieveAnswer>  answer_;
   uint64_t                networkSize_;
   bool                    debug_;
 
@@ -1344,7 +1427,7 @@ private:
   }
 
 
-  virtual void CancelFunction()
+  virtual void CancelFunction() ORTHANC_OVERRIDE
   {
     boost::mutex::scoped_lock lock(mutex_);
 
@@ -1355,13 +1438,13 @@ private:
     }
   }
 
-  virtual void PauseFunction()
+  virtual void PauseFunction() ORTHANC_OVERRIDE
   {
     // This type of job cannot be paused
     CancelFunction();
   }
 
-  virtual IFunction* CreateFunction()
+  virtual IFunction* CreateFunction() ORTHANC_OVERRIDE
   {
     // This type of job cannot be paused: If restarting, always go
     // back to the beginning
@@ -1442,7 +1525,7 @@ void WadoRetrieveClient(OrthancPluginRestOutput* output,
   Json::Value body;
   OrthancPlugins::ParseJsonBody(body, request);
 
-  std::auto_ptr<WadoRetrieveJob>  job(new WadoRetrieveJob(serverName));
+  std::unique_ptr<WadoRetrieveJob>  job(new WadoRetrieveJob(serverName));
   job->AddResourceFromRequest(body);
 
   bool debug;
@@ -1488,7 +1571,7 @@ void RetrieveFromServer(OrthancPluginRestOutput* output,
   std::map<std::string, std::string> additionalHeaders;
   OrthancPlugins::ParseAssociativeArray(additionalHeaders, body, HTTP_HEADERS);
 
-  std::auto_ptr<WadoRetrieveJob> job(new WadoRetrieveJob(serverName));
+  std::unique_ptr<WadoRetrieveJob> job(new WadoRetrieveJob(serverName));
 
   if (body.type() != Json::objectValue ||
       !body.isMember(RESOURCES) ||
