@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -24,7 +25,7 @@
 #include <ChunkedBuffer.h>
 #include <DicomFormat/DicomMap.h>
 
-#include <orthanc/OrthancCPlugin.h>
+#include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 
 #include <json/value.h>
 
@@ -73,6 +74,13 @@ namespace OrthancPlugins
                       OrthancPluginDicomWebBinaryMode mode,
                       const std::string& bulkRoot);
 
+    static void Apply(std::string& target,
+                      OrthancPluginContext* context,
+                      const DicomInstance& instance,
+                      bool xml,
+                      OrthancPluginDicomWebBinaryMode mode,
+                      const std::string& bulkRoot);
+
     class HttpWriter : public boost::noncopyable
     {
     private:
@@ -111,6 +119,9 @@ namespace OrthancPlugins
                                      size_t size);
 
       void Send();
+
+      void AddInstance(const DicomInstance& instance,
+                       const std::string& bulkRoot);
     };
   };
 }
