@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -35,6 +36,11 @@
 #include <boost/algorithm/string/predicate.hpp>
 
 
+#define ORTHANC_CORE_MINIMAL_MAJOR     1
+#define ORTHANC_CORE_MINIMAL_MINOR     11
+#define ORTHANC_CORE_MINIMAL_REVISION  0
+
+
 static const char* const HAS_DELETE = "HasDelete";
 
 
@@ -512,11 +518,30 @@ extern "C"
       return -1;
     }
 
+    if (!OrthancPlugins::CheckMinimalOrthancVersion(ORTHANC_CORE_MINIMAL_MAJOR,
+                                                    ORTHANC_CORE_MINIMAL_MINOR,
+                                                    ORTHANC_CORE_MINIMAL_REVISION))
+    {
+      char info[1024];
+      sprintf(info, "Your version of Orthanc (%s) must be above %d.%d.%d to run this plugin",
+              context->orthancVersion,
+              ORTHANC_CORE_MINIMAL_MAJOR,
+              ORTHANC_CORE_MINIMAL_MINOR,
+              ORTHANC_CORE_MINIMAL_REVISION);
+      OrthancPluginLogError(context, info);
+      return -1;
+    }
+
 #if HAS_ORTHANC_PLUGIN_CHUNKED_HTTP_CLIENT == 0
     LOG(WARNING) << "Performance warning in DICOMweb: The plugin was compiled against "
                  << "Orthanc SDK <= 1.5.6. STOW and WADO chunked transfers will be entirely stored in RAM.";
 #endif
 
+#if !ORTHANC_PLUGINS_VERSION_IS_ABOVE(1, 12, 1)
+    LOG(WARNING) << "Performance warning in DICOMweb: The plugin was compiled against "
+                 << "Orthanc SDK <= 1.12.0. Retrieving metadata will be slower.";
+#endif
+
     OrthancPluginSetDescription(context, "Implementation of DICOMweb (QIDO-RS, STOW-RS and WADO-RS) and WADO-URI.");
 
     try
@@ -608,6 +633,9 @@ extern "C"
         
         std::string uri = root + "app/client/index.html";
         OrthancPluginSetRootUri(context, uri.c_str());
+
+        std::string publicUrlRoot = OrthancPlugins::Configuration::GetPublicRoot();
+        OrthancPlugins::LogWarning("DICOMWeb PublicRoot: " + publicUrlRoot);
       }
       else
       {
